package com.xydl.service;

import com.xydl.entity.*;
import com.xydl.model.AssetList;
import com.xydl.util.DateUtil;
import com.xydl.util.JSONUtil;
import com.xydl.util.MqttUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;

@Service
@Slf4j
public class MqttService {

    @Resource
    MqttUtil mqttUtil;
    @Resource
    I2relationService i2relationService;
    @Resource
    SyncTablesInfoService tablesInfoService;
    @Resource
    SyncFieldsInfoService fieldsInfoService;
    @Resource
    SyncRecordsService recordsService;
    @Resource
    DbsqlService dbsqlService;

    // 上传任务
    public void uploadTask() {
        log.info("转发任务开始.");
        List<SyncTablesInfo> tableList = tablesInfoService.findAll();
        for (SyncTablesInfo table : tableList) {
            try {
                processOneTable(table);
            } catch (Exception e) {
                log.error("processOneTable exception: " + table.getId().getTableName(), e);
            }
        }
        log.info("转发任务结束.");
    }

    // 处理单个表
    private void processOneTable(SyncTablesInfo table) throws Exception {
        List<SyncFieldsInfo> fieldList = fieldsInfoService.findAll(table.getId().getClientId(), table.getId().getTableName());
        if (!CollectionUtils.isEmpty(fieldList)) {
            List<Object> eqmidList = dbsqlService.getDistinctEqmids(table.getId().getTableName(), table.getDevidFieldName());
            log.info("表{},共{}个设备", table.getId().getTableName(), eqmidList.size());
            if (!CollectionUtils.isEmpty(eqmidList)) {
                mqttUtil.connect();
                for (Object eqmid : eqmidList) {
                    this.processOneEqmid(eqmid, table, fieldList);
                }
                mqttUtil.disconnect();
            }
        } else {
            log.error("表{}未配置sync_fields_info", table.getId().getTableName());
        }
    }

    // 处理单个设备
    private void processOneEqmid(Object eqmid, SyncTablesInfo table, List<SyncFieldsInfo> fieldList) {

        List<I2relation> relationList = i2relationService.getList(eqmid);

        if (!CollectionUtils.isEmpty(relationList)) {
            try {
                int count = this.uploadOneBlock(eqmid, null,
                        table, fieldList, relationList);
                if (count > 1000) {
                    log.debug("表{}设备{}一共推送成功{}条数据.", table.getId().getTableName(), eqmid, count);
                }
            } catch (Exception e) {
                log.error("表{}设备{}推送异常", table.getId().getTableName(), eqmid, e);
            }
        } else {
            log.warn("表{}设备{}未配置i2relation", table.getId().getTableName(), eqmid);
        }
    }

    // 查询并上传单个设备的数据
    private int uploadOneBlock(Object eqmid, SyncRecords record,
                               SyncTablesInfo table, List<SyncFieldsInfo> fieldList, List<I2relation> relationList) throws Exception {

        if (record == null) {
            Optional<SyncRecords> optional = recordsService.findByKey(table.getId().getClientId(), table.getId().getTableName(), String.valueOf(eqmid));
            if (optional.isPresent()) {
                record = optional.get();
            } else {
                SyncRecordsKey key = SyncRecordsKey.builder()
                        .clientId(table.getId().getClientId())
                        .tableName(table.getId().getTableName())
                        .devidVal(String.valueOf(eqmid))
                        .build();
                record = SyncRecords.builder()
                        .id(key)
                        .build();
            }
        }

        int count = 0;
        List<Map<String, Object>> dataList = dbsqlService.getData(eqmid, record.getFieldVal2(), table);
        if (CollectionUtils.isEmpty(dataList)) {
            return 0;
        } else {
            count = dataList.size();
            AssetList assetList = AssetList.fromData(dataList, table.getFieldName(), relationList, fieldList);
            if (assetList != null) {
                String json = JSONUtil.object2Json(assetList);
                mqttUtil.publish2MQTT(json);
            }

            Map<String, Object> lastMap = dataList.get(count - 1);
            LocalDateTime locdate = (LocalDateTime) lastMap.get(table.getFieldName());
            Date date = Date.from(locdate.atZone(ZoneId.systemDefault()).toInstant());
            record.setFieldVal2(date);
            record.setUpdateTime(new Date());
            recordsService.update(record);

            log.debug("表{}设备{}推送成功{}条数据，最后时间{}", table.getId().getTableName(), eqmid, count, DateUtil.format(date));

            if (count >= 1000) {
                count += uploadOneBlock(eqmid, record,
                        table, fieldList, relationList);
            }
            return count;
        }
    }
}
